@@ -5,11 +5,11 @@
 
 #include <QFont>
 #include <QObject>
+#include <QQuickTextDocument>
 #include <QTextCursor>
 #include <QUrl>
 
 class QTextDocument;
-class QQuickTextDocument;
 class NeoChatRoom;
 class Controller;
 
