@@ -12,7 +12,6 @@
 #include <KWindowConfig>
 #ifdef HAVE_WINDOWSYSTEM
 #include <KWindowEffects>
-#include <KWindowSystem>
 #endif
 
 #include <QAuthenticator>
@@ -21,14 +20,15 @@
 #include <QFileInfo>
 #include <QGuiApplication>
 #include <QMovie>
-#include <QNetworkConfigurationManager>
 #include <QNetworkReply>
 #include <QPixmap>
 #include <QQuickItem>
+#include <QQuickTextDocument>
 #include <QQuickWindow>
 #include <QStandardPaths>
 #include <QStringBuilder>
 #include <QSysInfo>
+#include <QTextDocument>
 #include <QTimer>
 #include <utility>
 
@@ -40,18 +40,21 @@
 #include "neochataccountregistry.h"
 #endif
 
-#include "csapi/account-data.h"
-#include "csapi/content-repo.h"
-#include "csapi/joining.h"
-#include "csapi/logout.h"
-#include "csapi/profile.h"
-#include "events/eventcontent.h"
+#include <csapi/account-data.h>
+#include <csapi/content-repo.h>
+#include <csapi/joining.h>
+#include <csapi/logout.h>
+#include <csapi/profile.h>
+#include <events/eventcontent.h>
+#include <qt_connection_util.h>
+#include <settings.h>
+
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
 #include "roommanager.h"
-#include "settings.h"
 #include "utils.h"
+#include "windowcontroller.h"
 
 #include <KStandardShortcut>
 
@@ -123,19 +126,17 @@ Controller::Controller(QObject *parent)
 #endif
 }
 
-Controller::~Controller()
-{
-    for (auto c : AccountRegistry::instance().accounts()) {
-        c->saveState();
-    }
-}
-
 Controller &Controller::instance()
 {
     static Controller _instance;
     return _instance;
 }
 
+void Controller::showWindow()
+{
+    WindowController::instance().showAndRaiseWindow(QString());
+}
+
 inline QString accessTokenFileName(const AccountSettings &account)
 {
     QString fileName = account.userId();
@@ -573,7 +574,7 @@ void Controller::setActiveConnection(Connection *connection)
     m_connection = connection;
     if (connection != nullptr) {
         NeoChatConfig::self()->setActiveConnection(connection->userId());
-        connect(connection, &Connection::networkError, this, [this](QString message, QString details, int retriesTaken, int nextRetryInMilliseconds) {
+        connect(connection, &Connection::networkError, this, [this]() {
             if (!m_isOnline) {
                 return;
             }
@@ -594,21 +595,9 @@ void Controller::setActiveConnection(Connection *connection)
     Q_EMIT activeConnectionChanged();
 }
 
-void Controller::restoreWindowGeometry(QQuickWindow *window)
+void Controller::saveWindowGeometry()
 {
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup windowGroup(&dataResource, "Window");
-    KWindowConfig::restoreWindowSize(window, windowGroup);
-    KWindowConfig::restoreWindowPosition(window, windowGroup);
-}
-
-void Controller::saveWindowGeometry(QQuickWindow *window)
-{
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup windowGroup(&dataResource, "Window");
-    KWindowConfig::saveWindowPosition(window, windowGroup);
-    KWindowConfig::saveWindowSize(window, windowGroup);
-    dataResource.sync();
+    WindowController::instance().saveGeometry();
 }
 
 NeochatDeleteDeviceJob::NeochatDeleteDeviceJob(const QString &deviceId, const Omittable<QJsonObject> &auth)
@@ -686,13 +675,6 @@ void Controller::setBlur(QQuickItem *item, bool blur)
 #endif
 }
 
-void Controller::raiseWindow(QWindow *window)
-{
-#ifdef HAVE_WINDOWSYSTEM
-    KWindowSystem::activateWindow(window->winId());
-#endif
-}
-
 bool Controller::hasWindowSystem() const
 {
 #ifdef HAVE_WINDOWSYSTEM
@@ -701,3 +683,17 @@ bool Controller::hasWindowSystem() const
     return false;
 #endif
 }
+
+QString Controller::plainText(QQuickTextDocument *document) const
+{
+    return document->textDocument()->toPlainText();
+}
+
+bool Controller::encryptionSupported() const
+{
+#ifdef QUOTIENT_07
+    return Quotient::encryptionSupported();
+#else
+    return false;
+#endif
+}
