@@ -11,16 +11,17 @@
 
 class QKeySequences;
 
-#include "connection.h"
-#include "csapi/list_public_rooms.h"
-#include "room.h"
-#include "settings.h"
-#include "user.h"
+#include <connection.h>
+#include <csapi/list_public_rooms.h>
+#include <room.h>
+#include <settings.h>
+#include <user.h>
 
 class NeoChatRoom;
 class NeoChatUser;
 class TrayIcon;
-class QQuickWindow;
+class QWindow;
+class QQuickTextDocument;
 
 namespace QKeychain
 {
@@ -40,6 +41,7 @@ class Controller : public QObject
     Q_PROPERTY(bool supportSystemTray READ supportSystemTray CONSTANT)
     Q_PROPERTY(bool hasWindowSystem READ hasWindowSystem CONSTANT)
     Q_PROPERTY(bool isOnline READ isOnline NOTIFY isOnlineChanged)
+    Q_PROPERTY(bool encryptionSupported READ encryptionSupported CONSTANT)
 
 public:
     static Controller &instance();
@@ -93,11 +95,11 @@ public:
     Q_INVOKABLE void openOrCreateDirectChat(NeoChatUser *user);
 
     Q_INVOKABLE void setBlur(QQuickItem *item, bool blur);
-    Q_INVOKABLE void raiseWindow(QWindow *window);
+    Q_INVOKABLE QString plainText(QQuickTextDocument *document) const;
+    bool encryptionSupported() const;
 
 private:
     explicit Controller(QObject *parent = nullptr);
-    ~Controller() override;
 
     QPointer<Connection> m_connection;
     bool m_busy = false;
@@ -115,6 +117,7 @@ private:
 
 private Q_SLOTS:
     void invokeLogin();
+    void showWindow();
 
 Q_SIGNALS:
     void busyChanged();
@@ -134,17 +137,19 @@ Q_SIGNALS:
     void activeConnectionChanged();
     void aboutDataChanged();
     void passwordStatus(Controller::PasswordStatus _t1);
-    void showWindow();
     void userConsentRequired(QUrl url);
     void testConnectionResult(const QString &connection, bool usable);
     void isOnlineChanged(bool isOnline);
+    void keyVerificationRequest(int timeLeft, Connection *connection, const QString &transactionId, const QString &deviceId);
+    void keyVerificationStart();
+    void keyVerificationAccept(const QString &commitment);
+    void keyVerificationKey(const QString &sas);
 
 public Q_SLOTS:
     void logout(Quotient::Connection *conn, bool serverSideLogout);
     void changeAvatar(Quotient::Connection *conn, const QUrl &localFile);
     static void markAllMessagesAsRead(Quotient::Connection *conn);
-    void restoreWindowGeometry(QQuickWindow *);
-    void saveWindowGeometry(QQuickWindow *);
+    void saveWindowGeometry();
 };
 
 // TODO libQuotient 0.7: Drop
