@@ -9,10 +9,17 @@
 
 using namespace Quotient;
 
+namespace Quotient
+{
+class Connection;
+}
+
 class DevicesModel : public QAbstractListModel
 {
     Q_OBJECT
 
+    Q_PROPERTY(Connection *connection READ connection NOTIFY connectionChanged)
+
 public:
     enum Roles {
         Id,
@@ -31,6 +38,11 @@ public:
     Q_INVOKABLE void logout(int index, const QString &password);
     Q_INVOKABLE void setName(int index, const QString &name);
 
+    Connection *connection() const;
+
+Q_SIGNALS:
+    void connectionChanged();
+
 private:
     void fetchDevices();
     QVector<Quotient::Device> m_devices;
