@@ -5,8 +5,8 @@
 
 #include <QObject>
 
-#include "connection.h"
-#include "csapi/wellknown.h"
+#include <connection.h>
+#include <csapi/wellknown.h>
 
 using namespace Quotient;
 
