@@ -26,8 +26,8 @@
 #include <KDBusService>
 #endif
 #ifdef HAVE_WINDOWSYSTEM
-#include <kwindowsystem_version.h>
 #include <KWindowSystem>
+#include <kwindowsystem_version.h>
 #endif
 #include <KLocalizedContext>
 #include <KLocalizedString>
@@ -36,11 +36,16 @@
 #include "neochat-version.h"
 
 #ifdef QUOTIENT_07
-#include "accountregistry.h"
+#include <accountregistry.h>
 #else
 #include "neochataccountregistry.h"
 #endif
 
+#include <csapi/joining.h>
+#include <csapi/leaving.h>
+#include <networkaccessmanager.h>
+#include <room.h>
+
 #include "actionshandler.h"
 #include "blurhashimageprovider.h"
 #include "chatboxhelper.h"
@@ -49,13 +54,12 @@
 #include "collapsestateproxymodel.h"
 #include "commandmodel.h"
 #include "controller.h"
-#include "csapi/joining.h"
-#include "csapi/leaving.h"
 #include "customemojimodel.h"
 #include "devicesmodel.h"
 #include "emojimodel.h"
 #include "filetypesingleton.h"
 #include "joinrulesevent.h"
+#include "linkpreviewer.h"
 #include "login.h"
 #include "matriximageprovider.h"
 #include "messageeventmodel.h"
@@ -63,16 +67,21 @@
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
-#include "networkaccessmanager.h"
 #include "notificationsmanager.h"
 #include "publicroomlistmodel.h"
 #include "roomlistmodel.h"
 #include "roommanager.h"
 #include "sortfilterroomlistmodel.h"
+#include "sortfilterspacelistmodel.h"
+#include "spacehierarchycache.h"
+#include "urlhelper.h"
 #include "userdirectorylistmodel.h"
 #include "userlistmodel.h"
 #include "webshortcutmodel.h"
-#include <room.h>
+#include "windowcontroller.h"
+#ifdef QUOTIENT_07
+#include <keyverificationsession.h>
+#endif
 #ifdef HAVE_COLORSCHEME
 #include "colorschemer.h"
 #endif
@@ -92,20 +101,13 @@ class NetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
     }
 };
 
-#ifdef HAVE_WINDOWSYSTEM
-static void raiseWindow(QWindow *window)
+static QWindow *windowFromEngine(QQmlApplicationEngine *engine)
 {
-#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 91, 0)
-    KWindowSystem::updateStartupId(window);
-#else
-    if (KWindowSystem::isPlatformWayland()) {
-        KWindowSystem::setCurrentXdgActivationToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"));
-    }
-#endif
-    KWindowSystem::activateWindow(window->winId());
-    window->raise();
+    const auto rootObjects = engine->rootObjects();
+    auto *window = qobject_cast<QQuickWindow *>(rootObjects.first());
+    Q_ASSERT(window);
+    return window;
 }
-#endif
 
 #ifdef Q_OS_ANDROID
 Q_DECL_EXPORT
@@ -139,6 +141,7 @@ int main(int argc, char *argv[])
     font.setPointSize(10);
     app.setFont(font);
 #endif
+    KLocalizedString::setApplicationDomain("neochat");
 
     QGuiApplication::setOrganizationName("KDE");
 
@@ -162,7 +165,6 @@ int main(int argc, char *argv[])
     KAboutData::setApplicationData(about);
     QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
-
 #ifdef NEOCHAT_FLATPAK
     // Copy over the included FontConfig configuration to the
     // app's config dir:
@@ -175,6 +177,8 @@ int main(int argc, char *argv[])
 
     Login *login = new Login();
     ChatBoxHelper chatBoxHelper;
+    UrlHelper urlHelper;
+    SpaceHierarchyCache spaceHierarchyCache;
 
 #ifdef HAVE_COLORSCHEME
     ColorSchemer colorScheme;
@@ -185,16 +189,19 @@ int main(int argc, char *argv[])
 #endif
 
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Controller", &Controller::instance());
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "NotificationsManager", &NotificationsManager::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Clipboard", &clipboard);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Config", config);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "RoomManager", &RoomManager::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "FileType", &fileTypeSingleton);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "ChatBoxHelper", &chatBoxHelper);
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "UrlHelper", &urlHelper);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "EmojiModel", new EmojiModel(&app));
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "CommandModel", new CommandModel(&app));
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::AccountRegistry::instance());
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "SpaceHierarchyCache", &spaceHierarchyCache);
     qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
+    qmlRegisterType<ChatBoxHelper>("org.kde.neochat", 1, 0, "ChatBoxHelper");
     qmlRegisterType<ChatDocumentHandler>("org.kde.neochat", 1, 0, "ChatDocumentHandler");
     qmlRegisterType<RoomListModel>("org.kde.neochat", 1, 0, "RoomListModel");
     qmlRegisterType<KWebShortcutModel>("org.kde.neochat", 1, 0, "WebShortcutModel");
@@ -206,8 +213,11 @@ int main(int argc, char *argv[])
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
+    qmlRegisterType<SortFilterSpaceListModel>("org.kde.neochat", 1, 0, "SortFilterSpaceListModel");
     qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
+    qmlRegisterType<LinkPreviewer>("org.kde.neochat", 1, 0, "LinkPreviewer");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
+    qmlRegisterUncreatableType<PushNotificationState>("org.kde.neochat", 1, 0, "PushNotificationState", "ENUM");
     qmlRegisterUncreatableType<NeoChatRoomType>("org.kde.neochat", 1, 0, "NeoChatRoomType", "ENUM");
     qmlRegisterUncreatableType<UserType>("org.kde.neochat", 1, 0, "UserType", "ENUM");
 
@@ -221,18 +231,20 @@ int main(int argc, char *argv[])
     qRegisterMetaType<NeoChatUser *>("NeoChatUser*");
     qRegisterMetaType<GetRoomEventsJob *>("GetRoomEventsJob*");
     qRegisterMetaType<QMimeType>("QMimeType");
-
-#ifdef HAVE_WINDOWSYSTEM
-    qmlRegisterSingletonType<KWindowSystem>("org.kde.kwindowsystem.private", 1, 0, "KWindowSystem", [](QQmlEngine *, QJSEngine *) -> QObject * {
-        return KWindowSystem::self();
-    });
+#ifdef QUOTIENT_07
+#ifdef Quotient_E2EE_ENABLED
+    qRegisterMetaType<KeyVerificationSession *>("KeyVerificationSession*");
+    qmlRegisterUncreatableType<KeyVerificationSession>("org.kde.neochat", 1, 0, "KeyVerificationSession", {});
+    qRegisterMetaType<QVector<EmojiEntry>>("QVector<EmojiEntry>");
+#endif
 #endif
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     qRegisterMetaTypeStreamOperators<Emoji>();
+#endif
 
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
-    KLocalizedString::setApplicationDomain("neochat");
     QObject::connect(&engine, &QQmlApplicationEngine::quit, &app, &QCoreApplication::quit);
     engine.setNetworkAccessManagerFactory(new NetworkAccessManagerFactory());
 
@@ -272,16 +284,10 @@ int main(int argc, char *argv[])
                     [&engine](const QStringList &arguments, const QString &workingDirectory) {
                         Q_UNUSED(workingDirectory);
 
-                        // Raise windows
-                        const auto rootObjects = engine.rootObjects();
-                        for (auto obj : rootObjects) {
-                            auto view = qobject_cast<QQuickWindow *>(obj);
-                            if (view) {
-                                view->show();
-                                raiseWindow(view);
-                                return;
-                            }
-                        }
+                        QWindow *window = windowFromEngine(&engine);
+                        KWindowSystem::updateStartupId(window);
+
+                        WindowController::instance().showAndRaiseWindow(QString());
 
                         // Open matrix uri
                         if (arguments.isEmpty()) {
@@ -294,15 +300,11 @@ int main(int argc, char *argv[])
                         }
                     });
 #endif
-    const auto rootObjects = engine.rootObjects();
-    for (auto obj : rootObjects) {
-        auto view = qobject_cast<QQuickWindow *>(obj);
-        if (view) {
-            if (view->isVisible()) {
-                Controller::instance().restoreWindowGeometry(view);
-            }
-            break;
-        }
-    }
+
+    QWindow *window = windowFromEngine(&engine);
+
+    WindowController::instance().setWindow(window);
+    WindowController::instance().restoreGeometry();
+
     return app.exec();
 }
