@@ -1,9 +1,7 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- * SPDX-FileCopyrightText: 2019 Kitsune Ral <kitsune-ral@users.sf.net>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-FileCopyrightText: 2019 Kitsune Ral <kitsune-ral@users.sf.net>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include "matriximageprovider.h"
 
 #include <QDebug>
@@ -21,7 +19,11 @@ using Quotient::BaseJob;
 ThumbnailResponse::ThumbnailResponse(QString id, QSize size)
     : mediaId(std::move(id))
     , requestedSize(size)
-    , localFile(QStringLiteral("%1/image_provider/%2-%3x%4.png").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), mediaId, QString::number(requestedSize.width()), QString::number(requestedSize.height())))
+    , localFile(QStringLiteral("%1/image_provider/%2-%3x%4.png")
+                    .arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation),
+                         mediaId,
+                         QString::number(requestedSize.width()),
+                         QString::number(requestedSize.height())))
     , errorStr("Image request hasn't started")
 {
     if (requestedSize.isEmpty()) {
@@ -85,7 +87,7 @@ void ThumbnailResponse::prepareResult()
             errorStr.clear();
         } else if (job->error() == BaseJob::Abandoned) {
             errorStr = i18n("Image request has been cancelled");
-            qDebug() << "ThumbnailResponse: cancelled for" << mediaId;
+            // qDebug() << "ThumbnailResponse: cancelled for" << mediaId;
         } else {
             errorStr = job->errorString();
             qWarning() << "ThumbnailResponse: no valid image for" << mediaId << "-" << errorStr;
