@@ -55,6 +55,7 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[FormattedBodyRole] = "formattedBody";
     roles[AuthorIdRole] = "authorId";
     roles[MediaUrlRole] = "mediaUrl";
+    roles[VerifiedRole] = "verified";
     return roles;
 }
 
@@ -193,6 +194,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             refreshEventRoles(eventId, {ReactionRole, Qt::DisplayRole});
         });
+        connect(m_currentRoom, &Room::newFileTransfer, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferProgress, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferCompleted, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferFailed, this, &MessageEventModel::refreshEvent);
@@ -540,12 +542,17 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == MimeTypeRole) {
-        auto e = eventCast<const RoomMessageEvent>(&evt);
-        if (!e || !e->hasFileContent()) {
-            return QVariant();
+        if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
+            if (!e || !e->hasFileContent()) {
+                return QVariant();
+            }
+
+            return e->content()->fileInfo()->mimeType.name();
         }
 
-        return e->content()->fileInfo()->mimeType.name();
+        if (auto e = eventCast<const StickerEvent>(&evt)) {
+            return e->image().mimeType.name();
+        }
     }
 
     if (role == SpecialMarksRole) {
@@ -714,9 +721,9 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         for (auto r = row + 1; r < rowCount(); ++r) {
             auto i = index(r);
             if (data(i, SpecialMarksRole) != EventStatus::Hidden) {
-                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) != data(idx, EventTypeRole)
-                    || data(idx, TimeRole).toDateTime().msecsTo(data(i, TimeRole).toDateTime()) > 600000
-                    || data(idx, TimeRole).toDateTime().toLocalTime().date().day() != data(i, TimeRole).toDateTime().toLocalTime().date().day();
+                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) == "state"
+                    || data(i, TimeRole).toDateTime().msecsTo(data(idx, TimeRole).toDateTime()) > 600000
+                    || data(i, TimeRole).toDateTime().toLocalTime().date().day() != data(idx, TimeRole).toDateTime().toLocalTime().date().day();
             }
         }
 
@@ -786,10 +793,38 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
                 return m_currentRoom->makeMediaUrl(e->id(), e->content()->originalJson["url"].toString());
             }
         }
+
+        // Requires https://github.com/quotient-im/libQuotient/pull/570
+        // if (auto e = eventCast<const StickerEvent>(&evt)) {
+        //     return m_currentRoom->makeMediaUrl(e->id(), e->url());
+        // }
 #endif
+
+        // Construct link in the same form as urlToDownload as that function doesn't work for stickers
+        if (auto e = eventCast<const StickerEvent>(&evt)) {
+            auto url = QUrl(m_currentRoom->connection()->homeserver().toString() + "/_matrix/media/r0/download/" + e->url().toString().remove("mxc://"));
+            QUrlQuery q(url.query());
+            q.addQueryItem("allow_remote", "true");
+            url.setQuery(q);
+            return url;
+        }
+
         return m_currentRoom->urlToDownload(evt.id());
     }
 
+    if (role == VerifiedRole) {
+#ifdef QUOTIENT_07
+#ifdef Quotient_E2EE_ENABLED
+        if (evt.originalEvent()) {
+            auto encrypted = dynamic_cast<const EncryptedEvent *>(evt.originalEvent());
+            Q_ASSERT(encrypted);
+            return m_currentRoom->connection()->isVerifiedSession(encrypted->sessionId());
+        }
+        return false;
+#endif
+#endif
+    }
+
     return {};
 }
 
