@@ -6,7 +6,7 @@
 #include <QAbstractListModel>
 
 #include "neochatroom.h"
-#include "room.h"
+#include <room.h>
 
 class MessageEventModel : public QAbstractListModel
 {
@@ -48,6 +48,7 @@ public:
         // For debugging
         EventResolvedTypeRole,
         AuthorIdRole,
+        VerifiedRole,
         LastRole, // Keep this last
     };
     Q_ENUM(EventRoles)
