@@ -6,34 +6,46 @@
 #include <cmark.h>
 
 #include <QFileInfo>
-#include <QImageReader>
 #include <QMetaObject>
 #include <QMimeDatabase>
 #include <QTextDocument>
 #include <functional>
 
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
+#include <QMediaMetaData>
+#include <QMediaPlayer>
+#endif
+
 #include <qcoro/qcorosignal.h>
 #include <qcoro/task.h>
 
-#include "connection.h"
-#include "csapi/account-data.h"
-#include "csapi/content-repo.h"
-#include "csapi/leaving.h"
-#include "csapi/redaction.h"
-#include "csapi/room_state.h"
-#include "csapi/rooms.h"
-#include "csapi/typing.h"
-#include "events/accountdataevents.h"
-#include "events/reactionevent.h"
-#include "events/roomcanonicalaliasevent.h"
-#include "events/roommessageevent.h"
-#include "events/roompowerlevelsevent.h"
-#include "events/typingevent.h"
-#include "jobs/downloadfilejob.h"
+#include <connection.h>
+#include <csapi/account-data.h>
+#include <csapi/content-repo.h>
+#include <csapi/leaving.h>
+#include <csapi/pushrules.h>
+#include <csapi/redaction.h>
+#include <csapi/room_state.h>
+#include <csapi/rooms.h>
+#include <csapi/typing.h>
+#include <events/accountdataevents.h>
+#include <events/eventcontent.h>
+#include <events/reactionevent.h>
+#include <events/roomcanonicalaliasevent.h>
+#include <events/roomcreateevent.h>
+#include <events/roommessageevent.h>
+#include <events/roompowerlevelsevent.h>
+#include <events/typingevent.h>
+#include <jobs/downloadfilejob.h>
+#include <qt_connection_util.h>
+#include <user.h>
+#include <csapi/report_content.h>
+#include <events/eventcontent.h>
+
+#include "controller.h"
 #include "neochatconfig.h"
 #include "notificationsmanager.h"
 #include "stickerevent.h"
-#include "user.h"
 #include "utils.h"
 
 #include <KLocalizedString>
@@ -41,6 +53,7 @@
 NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinState)
     : Room(connection, std::move(roomId), joinState)
 {
+    connect(connection, &Connection::accountDataChanged, this, &NeoChatRoom::updatePushNotificationState);
     connect(this, &NeoChatRoom::notificationCountChanged, this, &NeoChatRoom::countChanged);
     connect(this, &NeoChatRoom::highlightCountChanged, this, &NeoChatRoom::countChanged);
     connect(this, &Room::fileTransferCompleted, this, [this] {
@@ -76,12 +89,38 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
 }
 
 void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
+{
+    doUploadFile(url, body);
+}
+
+QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body)
 {
     if (url.isEmpty()) {
-        return;
+        co_return;
     }
 
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
+    auto mime = QMimeDatabase().mimeTypeForUrl(url);
+    QFileInfo fileInfo(url.toLocalFile());
+    EventContent::TypedBase *content;
+    if (mime.name().startsWith("image/")) {
+        QImage image;
+        content = new EventContent::ImageContent(url, fileInfo.size(), mime, image.size(), fileInfo.fileName());
+    } else if (mime.name().startsWith("audio/")) {
+        content = new EventContent::AudioContent(url, fileInfo.size(), mime, fileInfo.fileName());
+    } else if (mime.name().startsWith("video/")) {
+        QMediaPlayer player;
+        player.setSource(url);
+        co_await qCoro(&player, &QMediaPlayer::mediaStatusChanged);
+        auto resolution = player.metaData().value(QMediaMetaData::Resolution).toSize();
+        content = new EventContent::VideoContent(url, fileInfo.size(), mime, resolution, fileInfo.fileName());
+    } else {
+        content = new EventContent::FileContent(url, fileInfo.size(), mime, fileInfo.fileName());
+    }
+    QString txnId = postFile(body.isEmpty() ? url.fileName() : body, content);
+#else
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, url, false);
+#endif
     setHasFileUploading(true);
 #ifdef QUOTIENT_07
     connect(this, &Room::fileTransferCompleted, [this, txnId](const QString &id, FileSourceInfo) {
@@ -602,7 +641,7 @@ void NeoChatRoom::removeLocalAlias(const QString &alias)
 QString NeoChatRoom::markdownToHTML(const QString &markdown)
 {
     const auto str = markdown.toUtf8();
-    char *tmp_buf = cmark_markdown_to_html(str.constData(), str.size(), CMARK_OPT_DEFAULT);
+    char *tmp_buf = cmark_markdown_to_html(str.constData(), str.size(), CMARK_OPT_HARDBREAKS);
 
     const std::string html(tmp_buf);
 
@@ -610,11 +649,9 @@ QString NeoChatRoom::markdownToHTML(const QString &markdown)
 
     auto result = QString::fromStdString(html).trimmed();
 
-    result.replace("<!-- raw HTML omitted -->", "<br />");
-    result.replace(QRegularExpression("(<br />)*$"), "");
+    result.replace("<!-- raw HTML omitted -->", "");
     result.replace("<p>", "");
     result.replace("</p>", "");
-    result.replace("\n", "<br>");
 
     return result;
 }
@@ -840,3 +877,220 @@ void NeoChatRoom::clearInvitationNotification()
 {
     NotificationsManager::instance().clearInvitationNotification(id());
 }
+
+bool NeoChatRoom::isSpace()
+{
+    const auto creationEvent = this->creation();
+    if (!creationEvent) {
+        return false;
+    }
+
+#ifdef QUOTIENT_07
+    return creationEvent->roomType() == RoomType::Space;
+#else
+    return false;
+#endif
+}
+
+void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
+{
+    // The caller should never try to set the state to unknown.
+    // It exists only as a default state to diable the settings options until the actual state is retrieved from the server.
+    if (state == PushNotificationState::Unknown) {
+        Q_ASSERT(false);
+        return;
+    }
+
+    /**
+     * This stops updatePushNotificationState from temporarily changing
+     * m_pushNotificationStateUpdating to default after the exisitng rules are deleted but
+     * before a new rule is added.
+     * The value is set to false after the rule enable job is successful.
+     */
+    m_pushNotificationStateUpdating = true;
+
+    /**
+     * First remove any exisiting room rules of the wrong type.
+     * Note to prevent race conditions any rule that is going ot be overridden later is not removed.
+     * If the default push notification state is chosen any exisiting rule needs to be removed.
+     */
+    QJsonObject accountData = connection()->accountDataJson("m.push_rules");
+
+    // For default and mute check for a room rule and remove if found.
+    if (state == PushNotificationState::Default || state == PushNotificationState::Mute) {
+        QJsonArray roomRuleArray = accountData["global"].toObject()["room"].toArray();
+        for (const auto &i : roomRuleArray) {
+            QJsonObject roomRule = i.toObject();
+            if (roomRule["rule_id"] == id()) {
+                Controller::instance().activeConnection()->callApi<DeletePushRuleJob>("global", "room", id());
+            }
+        }
+    }
+
+    // For default, all and @mentions and keywords check for an override rule and remove if found.
+    if (state == PushNotificationState::Default || state == PushNotificationState::All || state == PushNotificationState::MentionKeyword) {
+        QJsonArray overrideRuleArray = accountData["global"].toObject()["override"].toArray();
+        for (const auto &i : overrideRuleArray) {
+            QJsonObject overrideRule = i.toObject();
+            if (overrideRule["rule_id"] == id()) {
+                Controller::instance().activeConnection()->callApi<DeletePushRuleJob>("global", "override", id());
+            }
+        }
+    }
+
+    if (state == PushNotificationState::Mute) {
+        /**
+         * To mute a room an override rule with "don't notify is set".
+         *
+         * Setup the rule action to "don't notify" to stop all room notifications
+         * see https://spec.matrix.org/v1.3/client-server-api/#actions
+         *
+         * "actions": [
+         *      "don't_notify"
+         * ]
+         */
+        const QVector<QVariant> actions = {"dont_notify"};
+        /**
+         * Setup the push condition to get all events for the current room
+         * see https://spec.matrix.org/v1.3/client-server-api/#conditions-1
+         *
+         * "conditions": [
+         *      {
+         *          "key": "type",
+         *          "kind": "event_match",
+         *          "pattern": "room_id"
+         *      }
+         * ]
+         */
+        PushCondition pushCondition;
+        pushCondition.kind = "event_match";
+        pushCondition.key = "room_id";
+        pushCondition.pattern = id();
+        const QVector<PushCondition> conditions = {pushCondition};
+
+        // Add new override rule and make sure it's enabled
+        auto job = Controller::instance().activeConnection()->callApi<SetPushRuleJob>("global", "override", id(), actions, "", "", conditions, "");
+        connect(job, &BaseJob::success, this, [this]() {
+            auto enableJob = Controller::instance().activeConnection()->callApi<SetPushRuleEnabledJob>("global", "override", id(), true);
+            connect(enableJob, &BaseJob::success, this, [this]() {
+                m_pushNotificationStateUpdating = false;
+            });
+        });
+    } else if (state == PushNotificationState::MentionKeyword) {
+        /**
+         * To only get notifcations for @ mentions and keywords a room rule with "don't_notify" is set.
+         *
+         * Note -  This works becuase a default override rule which catches all user mentions will
+         * take precedent and notify. See https://spec.matrix.org/v1.3/client-server-api/#default-override-rules. Any keywords will also have a similar override
+         * rule.
+         *
+         * Setup the rule action to "don't notify" to stop all room event notifications
+         * see https://spec.matrix.org/v1.3/client-server-api/#actions
+         *
+         * "actions": [
+         *      "don't_notify"
+         * ]
+         */
+        const QVector<QVariant> actions = {"dont_notify"};
+        // No conditions for a room rule
+        const QVector<PushCondition> conditions;
+
+        auto setJob = Controller::instance().activeConnection()->callApi<SetPushRuleJob>("global", "room", id(), actions, "", "", conditions, "");
+        connect(setJob, &BaseJob::success, this, [this]() {
+            auto enableJob = Controller::instance().activeConnection()->callApi<SetPushRuleEnabledJob>("global", "room", id(), true);
+            connect(enableJob, &BaseJob::success, this, [this]() {
+                m_pushNotificationStateUpdating = false;
+            });
+        });
+    } else if (state == PushNotificationState::All) {
+        /**
+         * To send a notification for all room messages a room rule with "notify" is set.
+         *
+         * Setup the rule action to "notify" so all room events give notifications.
+         * Tweeks is also set to follow default sound settings
+         * see https://spec.matrix.org/v1.3/client-server-api/#actions
+         *
+         * "actions": [
+         *      "notify",
+         *      {
+         *          "set_tweek": "sound",
+         *          "value": "default",
+         *      }
+         * ]
+         */
+        QJsonObject tweaks;
+        tweaks.insert("set_tweak", "sound");
+        tweaks.insert("value", "default");
+        const QVector<QVariant> actions = {"notify", tweaks};
+        // No conditions for a room rule
+        const QVector<PushCondition> conditions;
+
+        // Add new room rule and make sure enabled
+        auto setJob = Controller::instance().activeConnection()->callApi<SetPushRuleJob>("global", "room", id(), actions, "", "", conditions, "");
+        connect(setJob, &BaseJob::success, this, [this]() {
+            auto enableJob = Controller::instance().activeConnection()->callApi<SetPushRuleEnabledJob>("global", "room", id(), true);
+            connect(enableJob, &BaseJob::success, this, [this]() {
+                m_pushNotificationStateUpdating = false;
+            });
+        });
+    }
+
+    m_currentPushNotificationState = state;
+    Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
+
+}
+
+void NeoChatRoom::updatePushNotificationState(QString type)
+{
+    if (type != "m.push_rules" || m_pushNotificationStateUpdating) {
+        return;
+    }
+
+    QJsonObject accountData = connection()->accountDataJson("m.push_rules");
+
+    // First look for a room rule with the room id
+    QJsonArray roomRuleArray = accountData["global"].toObject()["room"].toArray();
+    for (const auto &i : roomRuleArray) {
+        QJsonObject roomRule = i.toObject();
+        if (roomRule["rule_id"] == id()) {
+            QString notifyAction = roomRule["actions"].toArray()[0].toString();
+            if (notifyAction == "notify") {
+                m_currentPushNotificationState = PushNotificationState::All;
+                Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
+                return;
+            } else if (notifyAction == "dont_notify") {
+                m_currentPushNotificationState = PushNotificationState::MentionKeyword;
+                Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
+                return;
+            }
+        }
+    }
+
+    // Check for an override rule with the room id
+    QJsonArray overrideRuleArray = accountData["global"].toObject()["override"].toArray();
+    for (const auto &i : overrideRuleArray) {
+        QJsonObject overrideRule = i.toObject();
+        if (overrideRule["rule_id"] == id()) {
+            QString notifyAction = overrideRule["actions"].toArray()[0].toString();
+            if (notifyAction == "dont_notify") {
+                m_currentPushNotificationState = PushNotificationState::Mute;
+                Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
+                return;
+            }
+        }
+    }
+
+    // If neither a room or override rule exist for the room then the setting must be default
+    m_currentPushNotificationState = PushNotificationState::Default;
+    Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
+}
+
+void NeoChatRoom::reportEvent(const QString &eventId, const QString &reason)
+{
+    auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason);
+    connect(job, &BaseJob::finished, this, [this, job]() {
+        if (job->error() == BaseJob::Success) {
+            Q_EMIT positiveMessage(i18n("Report sent successfully."));
+        }
+    });
+}
