@@ -16,9 +16,13 @@
 #endif
 #include <KNotificationReplyAction>
 
+#include "csapi/pushrules.h"
+#include "jobs/basejob.h"
+
 #include "controller.h"
 #include "neochatconfig.h"
 #include "roommanager.h"
+#include "windowcontroller.h"
 
 NotificationsManager &NotificationsManager::instance()
 {
@@ -29,6 +33,10 @@ NotificationsManager &NotificationsManager::instance()
 NotificationsManager::NotificationsManager(QObject *parent)
     : QObject(parent)
 {
+    // Can't connect the signal up until the active connection has been established by the controller
+    connect(&Controller::instance(), &Controller::activeConnectionChanged, this, [this]() {
+        connect(Controller::instance().activeConnection(), &Connection::accountDataChanged, this, &NotificationsManager::updateGlobalNotificationsEnabled);
+    });
 }
 
 void NotificationsManager::postNotification(NeoChatRoom *room,
@@ -57,11 +65,8 @@ void NotificationsManager::postNotification(NeoChatRoom *room,
 
     notification->setDefaultAction(i18n("Open NeoChat in this room"));
     connect(notification, &KNotification::defaultActivated, this, [=]() {
-#if defined(HAVE_WINDOWSYSTEM) && KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 90, 0)
-        KWindowSystem::setCurrentXdgActivationToken(notification->xdgActivationToken());
-#endif
         RoomManager::instance().enterRoom(room);
-        Q_EMIT Controller::instance().showWindow();
+        WindowController::instance().showAndRaiseWindow(notification->xdgActivationToken());
     });
 
     if (canReply) {
@@ -75,7 +80,6 @@ void NotificationsManager::postNotification(NeoChatRoom *room,
 
     notification->setHint(QStringLiteral("x-kde-origin-name"), room->localUser()->id());
 
-
     notification->sendEvent();
 
     m_notifications.insert(room->id(), notification);
@@ -95,19 +99,16 @@ void NotificationsManager::postInviteNotification(NeoChatRoom *room, const QStri
     notification->setFlags(KNotification::Persistent);
     notification->setDefaultAction(i18n("Open this invitation in NeoChat"));
     connect(notification, &KNotification::defaultActivated, this, [=]() {
-#if defined(HAVE_WINDOWSYSTEM) && KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 90, 0)
-        KWindowSystem::setCurrentXdgActivationToken(notification->xdgActivationToken());
-#endif
         notification->close();
         RoomManager::instance().enterRoom(room);
-        Q_EMIT Controller::instance().showWindow();
+        WindowController::instance().showAndRaiseWindow(notification->xdgActivationToken());
     });
     notification->setActions({i18n("Accept Invitation"), i18n("Reject Invitation")});
-    connect(notification, &KNotification::action1Activated, this, [this, room, notification]() {
+    connect(notification, &KNotification::action1Activated, this, [room, notification]() {
         room->acceptInvitation();
         notification->close();
     });
-    connect(notification, &KNotification::action2Activated, this, [this, room, notification]() {
+    connect(notification, &KNotification::action2Activated, this, [room, notification]() {
         RoomManager::instance().leaveRoom(room);
         notification->close();
     });
@@ -127,3 +128,42 @@ void NotificationsManager::clearInvitationNotification(const QString &roomId)
         m_invitations[roomId]->close();
     }
 }
+
+/**
+ * The master push rule sets all notifications to off when enabled
+ * see https://spec.matrix.org/v1.3/client-server-api/#default-override-rules
+ * therefore to enable push rules the master rule needs to be disabled and vice versa
+ */
+void NotificationsManager::setGlobalNotificationsEnabled(bool enabled)
+{
+    using namespace Quotient;
+
+    auto job = Controller::instance().activeConnection()->callApi<IsPushRuleEnabledJob>("global", "override", ".m.rule.master");
+    connect(job, &BaseJob::success, this, [this, job, enabled]() {
+        if (job->enabled() == enabled) {
+            Controller::instance().activeConnection()->callApi<SetPushRuleEnabledJob>("global", "override", ".m.rule.master", !enabled);
+            m_globalNotificationsEnabled = enabled;
+            Q_EMIT globalNotificationsEnabledChanged(m_globalNotificationsEnabled);
+        }
+    });
+}
+
+void NotificationsManager::updateGlobalNotificationsEnabled(QString type)
+{
+    if (type != "m.push_rules") {
+        return;
+    }
+
+    QJsonObject accountData = Controller::instance().activeConnection()->accountDataJson("m.push_rules");
+    QJsonArray overrideRuleArray = accountData.value("global").toObject().value("override").toArray();
+
+    for (const auto &i : overrideRuleArray) {
+        QJsonObject overrideRule = i.toObject();
+        if (overrideRule.value("rule_id") == ".m.rule.master") {
+            bool ruleEnabled = overrideRule.value("enabled").toBool();
+            m_globalNotificationsEnabled = !ruleEnabled;
+            NeoChatConfig::self()->setShowNotifications(m_globalNotificationsEnabled);
+            Q_EMIT globalNotificationsEnabledChanged(m_globalNotificationsEnabled);
+        }
+    }
+}
