@@ -3,10 +3,11 @@
 
 #pragma once
 
-#include "connection.h"
-#include "events/roomevent.h"
+#include <connection.h>
+#include <events/roomevent.h>
+#include <room.h>
+
 #include "neochatroom.h"
-#include "room.h"
 
 #include <QAbstractListModel>
 
@@ -52,6 +53,7 @@ public:
         SubtitleTextRole,
         AvatarImageRole,
         IdRole,
+        IsSpaceRole,
     };
     Q_ENUM(EventRoles)
 
@@ -99,6 +101,7 @@ private:
     QMap<int, bool> m_categoryVisibility;
 
     int m_notificationCount = 0;
+    QString m_activeSpaceId = "";
 
     void connectRoomSignals(NeoChatRoom *room);
     void handleNotifications();
