@@ -3,8 +3,8 @@
 
 #pragma once
 
-#include "events/eventcontent.h"
-#include "events/roomevent.h"
+#include <events/eventcontent.h>
+#include <events/roomevent.h>
 
 namespace Quotient
 {
@@ -14,7 +14,11 @@ namespace Quotient
 class StickerEvent : public RoomEvent
 {
 public:
+#ifdef QUOTIENT_07
+    QUO_EVENT(StickerEvent, "m.sticker")
+#else
     DEFINE_EVENT_TYPEID("m.sticker", StickerEvent)
+#endif
 
     explicit StickerEvent(const QJsonObject &obj);
 
