@@ -75,7 +75,7 @@ void UserDirectoryListModel::search(int count)
 
     job = m_connection->callApi<SearchUserDirectoryJob>(m_keyword, count);
 
-    connect(job, &BaseJob::finished, this, [=] {
+    connect(job, &BaseJob::finished, this, [this] {
         attempted = true;
 
         if (job->status() == BaseJob::Success) {
@@ -126,8 +126,11 @@ QVariant UserDirectoryListModel::data(const QModelIndex &index, int role) const
         if (avatarUrl.isEmpty()) {
             return "";
         }
-
+#ifdef QUOTIENT_07
+        return avatarUrl.url().remove(0, 6);
+#else
         return avatarUrl.remove(0, 6);
+#endif
     }
     if (role == UserIDRole) {
         return user.userId;
